--[[
*********************************
LowLight.lua by Sinter, 5. Maerz 2010, Version 0.1
DANKE an rudi, msl und gehtnix fuer Eure hervorragende Grundlagenarbeit!


Anwendungsfelder: Verwackelungsschutz bei Freihandaufnahmen selbst bei schwachem Licht,
                  Available-Light-Fotografie,
                  Blitzvermeidung bei schwaecheren Lichtverhaeltnissen,
                  auch fr Tele-Aufnahmen konzipiert (da das Skript die Brennweite bercksichtigt).

mittels automatischer Bedarfsanalyse sowie (im Bedarfsfall) automatischer Auswahl und
Anwendung der 4 ISOBoost-Kurven bei automatischer Wahrung von mglichst kurzen Verschlusszeiten. 

Skript zur Beibehaltung moeglichst kurzer Verschlusszeiten (Ziel-Limit = abhngig von Brennweite) auch bei
schwachem Licht mittels automatischer Auswahl einer moeglichst passenden ISOBoost-Kurve
und entsprechender Belichtungszeitverkuerzung um bis zu 4 EV.

Das Skript misst ob aufgrund einer zu langen Belichtungszeit eine Verwackelung droht und 
verkrzt im Bedarfsfall ganz automatisch die Belichtungszeit unter Zuhilfenahme meiner ISOBoost-Kurven um 
bis zu 4 EV. Dies ermglicht Aufnahmen aus der Hand (bei ursprgl. Ziel = 1/40 Sekunde) sofern sie ursprnglich
bis zu 0,4 Sekunden Belichtungszeit bentigen wrden. (Erst ursprngliche Belichtungszeiten von 
lnger als 0,4 Sekunden werden in Belichtungszeiten von lnger
als 1/40 Sekunde umgewandelt, wobei dann die Gefahr von Verwackelungen deutlich zunimmt.)
Fr Kameras mit <=30mm-Weitwinkel habe ich auch noch einen 1/30-Sekunden-Zielwert beruecksichtig/eingefgt.

Die BelichtungszeitZielwerte (=minimapex) werden anhand der jeweils aktuellen Brennweite festgelegt.
Es koennen maximal 4 EV gewonnen werden. BelichtungszeitZielwerte, welche eine groessere Korrektur 
als die verfuegbaren 4 EV erfordern wuerden, werden nicht vollstndig realisiert (sondern dann nur
die Maximalkorrektur von 4 EV ausgefuehrt) und es erfolgt eine Warnmeldung, dass whrend der 
Aufnahme die Wackelgrenze ueberschritten war, sowie eine Aufforderung, um wie viele 
Stufen die ISO-Einstellung moeglichst erhoeht werden soll. (Praezision dieser Prognose noch
nicht geklaert. Theoretisch praezise, aber in der Praxis scheint ISO-Prognose unserer Cams etwas tueckisch,
nach erstens Tests vor allem bei der Prognose im Telebereich.)

##########################

!!! ZWINGENDE VORAUSSETZUNGEN !!!:
Das Skript funktioniert ausschlielich mit CHDK-10-Bit-RAW-Kameras und setzt
eine in CHDK geladene und unmittelbar (!) im CURVES-Ordner befindliche beliebige STD.CVF-Tonwertkurve
voraus, sowie an gleicher Stelle die vier ISOBoost-Kurven aus SintersCurvePackage. Die STD.CVF-Tonwertkurve muss
in den CHDK-Tonwertkurven-Einstellungen nicht aktiviert (Kurve darf AUS [=NONE] sein), aber muss
unbedingt geladen sein ("Current custom curve=A/CHDK/CURVES/STD.CVF").

Weitere Voraussetzung: CHDK-DE Version ab 26.2.2010

Anmerkung: Die vier ISO-Boost-Filenamen beginnen hier mit einem kleinen Buchstaben um das CURVES-Hauptverzeichnis
nicht zu sehr zu beeintraechtigen. Durch Kleinschreibung rutschen die Filenamen in der CHDK-Anzeige an das
Verzeichnisende. Die Kurve STD.CVF habe ich in Grossschrift belassen da die Kurve in dieser Form auch
bei Curve-Bracketing Verwendung findet.

##########################
[EDIT] Nicht mehr ntig:
Info zu einer Klassifikationsvariablen:

Klassifikation von Cam-Sensorgroessen fuer das Skript um aus 
get_focal_length die Aequivalenzbrennweite berechnen zu knnen:
1/2,5-Sensor => CCD-Sensorgroesse s=1  z. B. Ixu 60
1/1,8-Sensor => CCD-Sensorgroesse s=2  z. B. PowSho 610
1/1,3-Sensor => CCD-Sensorgroesse s=3  z. B. 
[/EDIT]
Stattdessen:

Infos zur Parametereinstellung: 
In den CHDK-Skripteinstellungen fuer die beiden folgenden Parameter bitte die eigenen (!) Kameradaten eintragen:

"Mini. CamDigiBrennweite x10" bedeutet die kleinste Millimeteranzahl des Brennweitenbereichs der Kamera (steht
vorne auf dem Objektiv), bitte multipliziert mit dem Faktor 10. (Bspw. bei "5,8mm" bitte hier "58" eintragen)

"Mini. CamAnalogBrennweite" bedeutet die kleinste dazu aequivalente Brennweite der Kamera im analogen 36mm-Format.
Das sind haeufig 35 Millimeter, bei Weitwinkelkameras oft auch 28 Millimeter. Den jeweiligen Wert kann man den
technischen Daten des eigenen Kamerahandbuchs entnehmen.

Diese beiden Daten verwendet das Skript zur Berechnung der jeweils aktuellen Brennweite im 
analogen 36mm-Format. Daraus wird dann gemaess Kehrwertregel die in der jeweiligen (Zoom-)Brennweiten-Einstellung
laengste tolerierbare Belichtungszeit gewaehlt, die gerade noch eine unverwackelte Freihand-Aufnahme zulaesst.

*********************************
@title LowLight by Sinter

@param a Rauschreduzierung    0=aus
@default a 0
@param e AmEnde ExitAlt  0=no 1=yes
@default e 0
@param d Min. CamDigiBrennweite x10
@default d 58
@param f Min. CamAnalogBrennweite
@default f 35

@param k Mehr Konsolenzeilen 0=aus
@default k 1
@param b Beobachtungs_Sleeps 0=aus
@default b 0

--]]


-- Plausibilittsprfung der Rauschreduzierung
if a<0 	then a=0 end
if a>1 	then a=1 end

-- Plausibilittsprfung der ExitAlt-Flag bei Beendigung des Skripts
if e<0 	then e=0 end
if e>1 	then e=1 end

-- Plausibilittsprfung der Beobachtungs_Sleeps -- Beobachtung dient nur zu Testzwecken/Debug
if b ~= 0 then b=1 end


-- Plausibilittsprfung der Sensorgroesse  #### inzwischen ueberfluessig
-- if s<1 	then s=1 end
-- if s>3 	then s=3 end




-- start script

-- Msste auch ohne folgende Startverzoegerung funktionieren
-- sleep(200)

cls()

-- Konsole evtl mit mehr Zeilen versehen
if k == 1 then set_console_layout(0,0,25,10) end



-- Rauschreduzierung: alten Status merken und gewuenschten Status aktivieren
rawnrstatusbeibeginn=get_raw_nr()
if a ~= 0 then set_raw_nr(2) end -- noise_reduction=ON
if a == 1 then set_raw_nr(1) end -- noise_reduction=OFF


-- Bestimmung der maximal tolerierbaren Belichtungszeit 
-- bei der man aus der Hand noch nicht verwackelt
-- anhand aktueller (Zoom-)Brennweite.
-- Entscheidung mittels Faustregel: Belichtungsdauer = hchstens Kehrwert der quivalenten Analogbrennweite.
-- Werte fr die Variablen f und d in den Formeln entnehmbar aus den technischen Daten der Kamera, =der 
-- digitalen Cam-Brennweitenbereiche und dem 36mm-Format-Cam-Aequivalenzbrennweitenbereiche

-- Eine verworfene Berechnungsalternative indem zuvor bei den Parametern die Sensorgroesse abgefragt wuerde:
-- if s == 1 then analogbrennweite=(get_focal_length()/100)*35/58        ## Diesen Weg
--   elseif s == 2 then analogbrennweite=(get_focal_length()/100)*35/73  ## verworfen, da Methode vermutlich
--   elseif s == 3 then analogbrennweite=(get_focal_length()/100)*39/54  ## nicht auf saemtliche Kameras
--   elseif s == 3 then analogbrennweite=(get_focal_length()/100)*28/58  ## uebertragbar. (Verschied. Brechung)
-- end                                                                   ## Werte wren offenbar nicht konsistent.
-- Daher eine geeignetere Berechnungsalternative bevorzugt:
analogbrennweite=(get_focal_length()/100)*f/d  -- 100 statt 1000 da d im Skript mit Faktor 10 genutzt wird.
if analogbrennweite <= 99 then print("Analogbrennweite ...", analogbrennweite)  -- nur wg Formatierung
   else print("Analogbrennweite ..=", analogbrennweite)
end
if b == 1 then sleep (3000) end


-- Um Verwackelungen zu vermeiden muss eine Zieluntergrenze fr die Belichtungszeit bestimmt werden:
-- Berechnung des mindestens ntigen Tv96-Apex-Wertes anhand der analogen Aequivalenzbrennweite
-- Beginn mit minimapex=480  d. h. = 1/30 Sekunde fuer Cams in 28 mm Weitwinkelstellung
if analogbrennweite <= 30 then minimapex=480          -- 1/30 Sekunde als minimal tolerierte Verschlusszeit
   elseif analogbrennweite <= 40 then minimapex=512   -- 1/40
   elseif analogbrennweite <= 50 then minimapex=544   -- 1/50
   elseif analogbrennweite <= 60 then minimapex=576   -- 1/60
   elseif analogbrennweite <= 80 then minimapex=608   -- 1/80
   elseif analogbrennweite <= 100 then minimapex=640  -- 1/100
   elseif analogbrennweite <= 125 then minimapex=672  -- 1/125
   elseif analogbrennweite <= 160 then minimapex=704  -- 1/160
   elseif analogbrennweite <= 200 then minimapex=736  -- 1/200
   elseif analogbrennweite > 200 then minimapex=768   -- frei gewaehlte maximale Untergrenze = 1/250 Sekunde
end

print("daher MinimApex: ...",minimapex)
if b == 1 then sleep (3000) end


-- Messung starten, wie die Kamera unter gegebenen
-- Bedingungen belichten MOECHTE (!), aber (falls Wackelrisiko droht) nicht unbedingt SOLL.
press("shoot_half") 
repeat 
  until get_shooting() == true 
release("shoot_half")
-- Urspruenglich notwendige Belichtungszeit abfragen und abspeichern
kamerawillbelichten=get_tv96()

-- Eventuell krumme Werte bereinigen und auf ein Vielfaches von 32 bringen.
-- Das macht Sinn falls gemessener TV-APEX-Wert knapp unter einem 32-er-Vielfachen
-- liegt und dadurch unntig evtl. eine um eine Stufe zu hohe ISOBoost-Kurve ausloesen wuerde. 
kamerawillbelichten=(kamerawillbelichten+16)/32
kamerawillbelichten=kamerawillbelichten*32
print ("Cam Tv-Apex wollte .", kamerawillbelichten)
-- sleep (2000)

-- Kleinste tolerierbare Belichtungszeit= abhaengig von AequivalenzBrennweite = als APEX-tv96-Wert = minimapex
-- Berechnung welche ISOBoost-Kurve zur Anwendung kommen soll:
-- belichtungsueberschuss (= wieviel lnger als toleriert(tv96Apex=minimapex) moechte die Cam ursprgl belichten) 
belichtungsueberschuss=minimapex-kamerawillbelichten
if belichtungsueberschuss >= 0 then print("ergibt Differenz von", belichtungsueberschuss) end
if belichtungsueberschuss < 0 then print("ergibt Differenz von", belichtungsueberschuss) end


-- Pruefen ob man die Anwendung von ISOBoost vermeiden kann. Falls JA, dann soll mit dem
-- gemessenen Wert und ohne ISOBoost-Kurve belichtet werden.
if belichtungsueberschuss <= 0 then
--   print("Es ging OHNE ISOBoost.")
   print("ISOBoost nicht bentigt.")
   wert=kamerawillbelichten
   -- Sicherheitshalber Kurven nochmals deaktivieren
   set_curve_state(0)
end


-- Quellkurvenmerker initialisieren
quellkurve=0



-- IsoBoost4 nutzen falls noetig
if belichtungsueberschuss>(3*96) then
   print("Anwendung von  ISOBoost4")
   -- Merke welche ISOBoost-Kurve nach der Anwendung wieder zurueck-"renamed" werden muss:
   quellkurve=4
   -- Variable zuruecksetzen damit weitere if-Abfragen bzgl niedrigerer ISOBoost-Kurven ins Leere laufen:
   belichtungsueberschuss=0
   -- RAW evtl EINschalten
   -- set_raw(1)
   -- HIER MUSS DIE ISOBOOST4-Kurve geladen (auf die STD.CVF kopiert) werden ###############################
   -- ISOBoost-Kurve wird zur STD.CVF renamed.  Vorher STD.CVF als STD_.CVF gesichert.
   os.rename("A/CHDK/CURVES/STD.CVF","A/CHDK/CURVES/STD_.CVF")
   os.rename("A/CHDK/CURVES/iB4EV.CVF","A/CHDK/CURVES/STD.CVF")
   -- Tonwertkurve Custom aktivieren. Dies darf erst NACH den
   -- rename-Prozessen erfolgen, sonst Bild oft zu hell. Grund??vielleicht alte Kurve?
   set_curve_state(1)
   -- Belichtung entsprechend IsoBoost-Kurve verkrzen:
   wert=(kamerawillbelichten+(96*4))
end



-- IsoBoost3 nutzen falls noetig
if belichtungsueberschuss>(2*96) then
   print("Anwendung von  ISOBoost3")
   quellkurve=3
   -- Variable zuruecksetzen damit weitere if-Abfragen ins Leere laufen:
   belichtungsueberschuss=0
   -- RAW evtl EINschalten
   -- set_raw(1)
   -- HIER MUSS DIE ISOBOOST3-Kurve geladen (auf die STD.CVF kopiert) werden ###############################
   os.rename("A/CHDK/CURVES/STD.CVF","A/CHDK/CURVES/STD_.CVF")
   os.rename("A/CHDK/CURVES/iB3EV.CVF","A/CHDK/CURVES/STD.CVF")
   -- Tonwertkurve Custom aktivieren.
   set_curve_state(1)
   -- Belichtung entsprechend IsoBoost-Kurve verkrzen:
   wert=(kamerawillbelichten+(96*3))
end



-- IsoBoost2 nutzen falls noetig
if belichtungsueberschuss>(1*96) then
   print("Anwendung von  ISOBoost2")
   quellkurve=2
   -- Variable zuruecksetzen damit weitere if-Abfragen ins Leere laufen:
   belichtungsueberschuss=0
   -- RAW evtl EINschalten
   -- set_raw(1)
   -- HIER MUSS DIE ISOBOOST2-Kurve geladen (auf die STD.CVF kopiert) werden ###############################
   os.rename("A/CHDK/CURVES/STD.CVF","A/CHDK/CURVES/STD_.CVF")
   os.rename("A/CHDK/CURVES/iB2EV.CVF","A/CHDK/CURVES/STD.CVF")
   -- Tonwertkurve Custom aktivieren.
   set_curve_state(1)   
   -- Belichtung entsprechend IsoBoost-Kurve verkrzen:
   wert=(kamerawillbelichten+(96*2))
end



-- IsoBoost1 nutzen falls noetig
if belichtungsueberschuss>(0*96) then
   print("Anwendung von  ISOBoost1")
   quellkurve=1
   -- Variable zuruecksetzen damit weitere if-Abfragen ins Leere laufen:
   belichtungsueberschuss=0
   -- RAW evtl EINschalten
   -- set_raw(1)
   -- HIER MUSS DIE ISOBOOST1-Kurve geladen (auf die STD.CVF kopiert) werden ###############################
   os.rename("A/CHDK/CURVES/STD.CVF","A/CHDK/CURVES/STD_.CVF")
   os.rename("A/CHDK/CURVES/iB1EV.CVF","A/CHDK/CURVES/STD.CVF")
   -- Tonwertkurve Custom aktivieren.
   set_curve_state(1)
   -- Belichtung entsprechend IsoBoost-Kurve verkrzen:
   wert=(kamerawillbelichten+(96*1))
end

-- sleep(1000)

-- Den fuer die Aufnahme realisierten BelichtungsApexWert anzeigen
print("realisiert. Tv-Apex:", wert)

-- SHOOTING:
-- Belichtungszeit setzen
-- set_tv96(wert)   oder vielleicht besser:
set_tv96_direct(wert)

press("shoot_half")
repeat
  until get_shooting() == true
-- sleep(1000)

press("shoot_full") 
release("shoot_full") 
release("shoot_half") 
repeat 
  sleep(1)
  until get_shooting() ~= true


-- Handling eines eventuell verbleibenden Rest-Verwackelungsrisikos:
-- Warnmeldung falls eigentlich eine Verschlusszeitkorrektur groesser als 4 EV noetig gewesen waere,
-- die jedoch mittels ISOBoost in diesem Umfang nicht mehr vollstndig realisierbar war/ist und
-- dadurch das Verwackelungsrisiko "nur" im Umfang der maximal erzielbaren 4 EV abgefangen werden konnte.
-- Folglich verbleibt in solchen Faellen ein Rest-Verwackelungsrisiko, welches der User durch 
-- Erhoehung (falls moeglich) seiner ISO-Einstellung evtl. beseitigen koennte.
-- Die berechnete Prognose ist wohl theoretisch praezise, indes scheint die Praxis bei ISO noch tueckisch zu sein.
if (minimapex-kamerawillbelichten) > (4*96) then
  verwackelungsrisiko=(((minimapex-kamerawillbelichten)-(4*96))+64)/96  -- "64" da jeweils 32-er-Vielfache
  verwriskganzzahl=((minimapex-kamerawillbelichten)-(4*96))/96
  verwriskmodulo=((minimapex-kamerawillbelichten)-(4*96))%96
--  print("RISIKOVERBLEIB", verwriskganzzahl, verwriskmodulo,"/3 EV") --Wie bekomme ich unntiges Blank vor "/" weg?
  print("RISIKOVERBLEIB ca.",verwackelungsrisiko,"EV !")
  print("! Falls mglich bitte ISO")
  print("um",verwackelungsrisiko, "Stufen erhhen!!")
  if b == 1 then sleep (3000) end
end


-- set_raw(0)


-- Rauschreduzierung auf den urspruenglichen Status zuruecksetzen
set_raw_nr(rawnrstatusbeibeginn) -- noise_reduction=wie bei Beginn


-- Verwendung der Tonwertkurve wieder deaktivieren
set_curve_state(0)


-- Genutzte Tonwertkurvenfiles zurueckrenamen
if quellkurve == 1 then
  os.rename("A/CHDK/CURVES/STD.CVF","A/CHDK/CURVES/iB1EV.CVF")
  os.rename("A/CHDK/CURVES/STD_.CVF","A/CHDK/CURVES/STD.CVF")
end

if quellkurve == 2 then
  os.rename("A/CHDK/CURVES/STD.CVF","A/CHDK/CURVES/iB2EV.CVF")
  os.rename("A/CHDK/CURVES/STD_.CVF","A/CHDK/CURVES/STD.CVF")
end

if quellkurve == 3 then
  os.rename("A/CHDK/CURVES/STD.CVF","A/CHDK/CURVES/iB3EV.CVF")
  os.rename("A/CHDK/CURVES/STD_.CVF","A/CHDK/CURVES/STD.CVF")
end

if quellkurve == 4 then
  os.rename("A/CHDK/CURVES/STD.CVF","A/CHDK/CURVES/iB4EV.CVF")
  os.rename("A/CHDK/CURVES/STD_.CVF","A/CHDK/CURVES/STD.CVF")
end

quellkurve=0

if e == 1 then -- Bei Beendigung des Skriptes den ALT-Modus verlassen
              exit_alt()
end


-- Manchmal werden bei Skriptende auf der Konsole Teile der Print-Texte gelscht/nicht mehr vollstaendig angezeigt.
-- Ursache? Hat jemand einen Tipp?
-- Fragwrdiger Versuch Gegenmanahme:
sleep(10)



-- Ungenutzt; nur als Shooting-Muster:
-- functions


function fastshoot()
  	press("shoot_half")
  	repeat
    	sleep(1)
  	until get_shooting() == true
  	press("shoot_full")
  	release("shoot_full")
  	release("shoot_half")
  	repeat
   	 	sleep(1)
  	until get_shooting() ~= true
end


--//////////////////////////////////






